#!/usr/bin/env python

"""Module Docstring"""

__author__ = "Camellia Magness"
__email__ = "cmagness@stsci.edu"

import os
import sys
import glob
import logging

import numpy as np
from matplotlib import rc
from astropy import constants
import matplotlib.pyplot as plt
from astropy.convolution import Box1DKernel, convolve

from . import SETTINGS

INPUTS = SETTINGS["inputs"]
DATADIR = INPUTS["datadir"]
OUTDIR = INPUTS["outdir"]
PARAMETERS = SETTINGS["parameters"]
DEFAULTS = SETTINGS["defaults"]
LOGGER = logging.getLogger(__name__)
console = logging.StreamHandler()
console.setLevel(logging.INFO)
LOGGER.addHandler(console)

C = constants.c.to('km/s').value  # km/s

N = 3.768e14  # proportionality constant -> (m_e * c)/(pi * e**2)


# --------------------------------------------------------------------------- #


class Visualizer:

    def __init__(self):
        self._target = None
        self._raw_flux = None
        self._raw_velocity = None
        self._lsr_velocity = None
        self._contadjspec = None  # this is the continuum adjusted spectrum
        self._left_indices = None
        self._right_indices = None
        self._helper = None

    def set_target(self, value):
        self._target = value

    def set_raw_flux(self, value):
        self._raw_flux = value

    def set_raw_velocity(self, value):
        self._raw_velocity = value

    def set_lsr_velocity(self, value):
        self._lsr_velocity = value

    def set_contadjspec(self, value):
        self._contadjspec = value

    def set_indices(self, left_value, right_value):
        self._left_indices = left_value
        self._right_indices = right_value

    def set_helper(self, value):
        self._helper = value

    def plot(self, box=5, show=DEFAULTS["show_plot"]):
        target = self._target
        raw_flux = self._raw_flux
        # at this point a list of arrays
        raw_velocities = self._raw_velocity
        # also a list of arrays
        lsr_velocities = self._lsr_velocity
        contadjspec = self._contadjspec
        helper = self._helper
        cont_left = DEFAULTS["continuum_left"]
        cont_right = DEFAULTS["continuum_right"]
        left = self._left_indices
        right = self._right_indices
        vel_min = PARAMETERS["vel_min"]
        vel_max = PARAMETERS["vel_max"]

        # you can't just do list membership, i.e. if None in <list>,
        # here because there are arrays in the list and that somehow messes
        # this up
        if any(elem is None for elem in [raw_flux, raw_velocities,
                                         lsr_velocities, contadjspec,
                                         helper]):
            LOGGER.error("NoneType in list: \n"
                         "Raw Flux: {} \n"
                         "Raw Velocity: {} \n"
                         "LSR Corrected Velocity: {} \n"
                         "Continuum Adjusted Spectrum: {} \n"
                         "Helper Object: {}".format(type(raw_flux),
                                                    type(raw_velocities),
                                                    type(lsr_velocities),
                                                    type(contadjspec),
                                                    type(helper)))
            LOGGER.error("Missing elements necessary to "
                         "plot. Exiting...")
            sys.exit()

        plt.rc('text', usetex=True)
        plt.rc('font', family='serif')
        plt.rc('axes', axisbelow=True)

        for idx, ion in enumerate(helper.ions["ion"]):
            wv = helper.ions["wv"][idx]
            # need to extract the velocities that correspond to the correct ion
            raw_velocity = raw_velocities[idx]
            lsr_velocity = lsr_velocities[idx]
            fig, ax = plt.subplots(2, 1, figsize=(15, 10), gridspec_kw={
                'height_ratios': [2, 1]
                })
            fig.suptitle("Visualization for {}: {}-{}"
                         .format(target, ion, int(wv)),
                         fontsize=20)
            ax[0].set_xlim(-500, 500)
            ax[1].set_xlim(-500, 500)

            max_flux = max(raw_flux[left[idx][0]:right[idx][1]])
            ax[0].set_ylim(0, 2 * max_flux)
            # scaling the axis based on the maximum value from lower left
            # continuum bound to upper right continuum bound

            kernel = Box1DKernel(box)
            boxcar_flux = convolve(raw_flux, kernel)

            ax[0].plot(raw_velocity, boxcar_flux, color="cornflowerblue",
                       linestyle=":", alpha=1, label="1. Original Flux")

            for xvel in [cont_left[0], cont_left[1], cont_right[0],
                         cont_right[1]]:
                ax[0].axvline(xvel, color="darkblue", linestyle="-.", alpha=1)

            ax[0].plot(lsr_velocity[left[idx][0]:left[idx][1] + 1],
                       contadjspec.continuum[idx]
                       [left[idx][0]:left[idx][1] + 1],
                       color="darkblue", linestyle="-.", alpha=1,
                       label="2. Continuum Windows/Fits")
            ax[0].plot(lsr_velocity[right[idx][0]:right[idx][1] + 1],
                       contadjspec.continuum[idx]
                       [right[idx][0]:right[idx][1] + 1],
                       color="darkblue", linestyle="-.", alpha=1)

            flux_l = np.mean(raw_flux[left[idx][0]:left[idx][1]])
            flux_r = np.mean(raw_flux[right[idx][0]:right[idx][1]])
            mean_flux = np.mean([flux_l, flux_r])
            # norm = contadjspec.norm_flux[0] * 1e-14
            adj_delta = 0.25 * max_flux  # offset value, should make 0.25
            # scaling an option instead of hardcoded
            # filter out any flux values in the normalized flux that are less
            # than 0
            mask = np.where(contadjspec.norm_flux[idx] > 0)
            norm_filtered = contadjspec.norm_flux[idx][mask]
            norm_convolved = convolve(norm_filtered, kernel)
            lsr_filtered = lsr_velocity[mask]
            norm = norm_convolved * mean_flux + adj_delta
            # find the indices where the
            ax[0].step(lsr_filtered, norm, where="mid", data=None,
                       color="royalblue",
                       label=r"3. Normalized Flux ($\times {"r":.2e} + {"
                             r":.2e}$)".format(mean_flux, adj_delta))
            hflux_convolved = convolve(helper.flux[idx], kernel)
            hflux = hflux_convolved * mean_flux + adj_delta
            ax[0].step(helper.velocity[idx], hflux, where="mid", data=None,
                       color="darkblue", alpha=1,
                       label="4. Measurement Window")

            for xvel in [vel_min, vel_max]:
                ax[0].axvline(xvel, color="darkblue", linestyle="-", alpha=1)
                ax[1].axvline(xvel, color="darkblue", linestyle="-", alpha=1)

            acd = helper.acd[idx]["acd"]
            max_acd = max(acd)
            ax[1].set_ylim(0, 1.5 * max_acd)  # 1.5 is just a scaling factor to
            # make the plot prettier
            ax[1].bar(helper.velocity[idx], acd, width=helper.delta[idx],
                      color="maroon", label=r'5. Measured $N_{a}(v)$')

            ax[0].set_xlabel(r'\textbf{Velocity ($km/s$)}', fontsize=15)
            ax[0].xaxis.set_label_position('top')
            ax[0].xaxis.tick_top()
            ax[0].set_ylabel(r'\textbf{Flux ($ergs/cm^{2}/s/\AA$)}',
                             color="darkblue", fontsize=15)
            ax[1].set_xlabel(r'\textbf{Velocity ($km/s$)}', fontsize=15)
            ax[1].set_ylabel(r'\textbf{$N_{a}(v)$ ($ions/cm^{2}/(km/s)$)}',
                color="maroon", fontsize=15)

            ax[0].legend(loc="upper left", fontsize=12)
            ax[1].legend(loc="lower right", fontsize=12)

            ax[0].tick_params(axis="both", which="major", labelsize=12)
            ax[1].tick_params(axis="both", which="major", labelsize=12)

            ax[0].grid(alpha=0.4, linestyle="--", linewidth=1)
            ax[1].grid(alpha=0.4, linestyle="--", linewidth=1)

            if len(target) < 10:
                target_string = target
            else:
                target_string = target[0:10]

            outfile = OUTDIR + "{}_{}{}_visualization.png".format(
                                       target_string, ion, int(wv))

            if os.path.exists(outfile):
                os.remove(outfile)
                # it's getting corrupted if i don't do this?

            plt.savefig(os.path.join(outfile))
            LOGGER.info("Visualization {}/{} has been saved to {}.".format(
                idx + 1, len(helper.ions["ion"]), outfile))

            # once plt.show() is called it clears the figure so it has to be
            # last
            if show:
                plt.show()

        LOGGER.info("Visualization module complete. ")


# --------------------------------------------------------------------------- #



#!/usr/bin/env python

"""This module is for measuring the apparent optical depth of absorption lines
in COS Fermi Bubble spectra. The method of measurement is outlined in Savage &
Sembach 1991. Eventually, this will be extended to measure apparent optical
depth of other spectra as well."""

__author__ = "Camellia Magness"
__email__ = "cmagness@stsci.edu"

import sys
import argparse
import logging

from . import SETTINGS
from .format_data import build_spectrum
from .spectrum_classes import Helper
from .visualization import Visualizer

INPUTS = SETTINGS["inputs"]
DATADIR = INPUTS["datadir"]
TARGETS = INPUTS["targets"]
PARAMETERS = SETTINGS["parameters"]
DEFAULTS = SETTINGS["defaults"]
BATCH_MODE = DEFAULTS["batch_table"]

# set up log file. will overwrite with each new run
# log file has DEBUG level + written to it, package, module, function
# console has only INFO level + written, basic format
logging.basicConfig(filename="logfile.log",
                    format="%(levelname)-4s[%(module)s.%(funcName)s.%("
                           "lineno)d]:%(message)s", filemode="w",
                    level=logging.DEBUG)
LOGGER = logging.getLogger(__name__)
console = logging.StreamHandler()
console.setLevel(logging.INFO)
LOGGER.addHandler(console)


# --------------------------------------------------------------------------- #


def main():
    # create the visualizer object
    visualizer = Visualizer()
    # argparse
    args, spectrum = parse()
    LOGGER.info("Spectrum object successfully built.")
    visualizer.set_target(spectrum.target)
    visualizer.set_raw_flux(spectrum.flux)
    # LSR correction
    spectrum = lsr_correct(args, spectrum)
    LOGGER.info("Spectrum LSR corrected.")
    visualizer.set_raw_velocity(spectrum.raw_velocity)
    visualizer.set_lsr_velocity(spectrum.velocity)
    # continuum fit
    spectrum, left_indices, right_indices = continuum_fit(spectrum)
    LOGGER.info("Continuum fit calculated.")
    visualizer.set_contadjspec(spectrum)
    visualizer.set_indices(left_indices, right_indices)
    # measure aod/acd/ew
    # set measurements back in spectrum object from helper object
    spectrum, helper = measure(args, spectrum)
    visualizer.set_helper(helper)
    # generate table
    spectrum.generate_table(args.vel_min, args.vel_max)
    visualizer.plot()
    LOGGER.info("spectrAOD complete.")
    return 0


# --------------------------------------------------------------------------- #


def parse():
    """This function is for parsing the inputs of interest."""

    parser = argparse.ArgumentParser(description="spectrAOD")

    # for debug only
    # parser.add_argument("--ion", default="SiIV", type=str,
    #                     help="absorption line feature of interest")

    parser.add_argument("--ion", default=PARAMETERS["ion"], type=str,
                        help="absorption line feature of interest")

    parser.add_argument("--instrument", default=PARAMETERS["instrument"],
                        type=str, help="observational instrument data is taken"
                                       "on")
    parser.add_argument("--filetype", default=PARAMETERS["filetype"], type=str,
                        help="filetype of data, takes X1DSUM (for x1ds or "
                             "x1dsums), BART, and BART-N")
    parser.add_argument("--vel_min", default=PARAMETERS["vel_min"], type=int,
                        help="velocity minimum in window of interest around "
                             "ion, in km/s")
    parser.add_argument("--vel_max", default=PARAMETERS["vel_max"], type=int,
                        help="velocity maximum in window of interest around "
                             "ion, in km/s")
    parser.add_argument("--grating", default=PARAMETERS["grating"], type=str,
                        help="grating of observation mode")
    parser.add_argument("--redshift", default=PARAMETERS["redshift"],
                        type=float, help="redshift of wavelength "
                                         "region/target")
    # need to do something about the grating bc not every dataset will have
    # a grating in the future

    # need to set up better debugging system than defaults. debugging flag?
    # need to set default grating to something for creating Spectrum object
    # NEED TO DO SOMETHING ABOUT THE FACT THAT COULD HAVE ION LIST IN THE
    # FUTURE

    # grating evaluation needs some work because it will have to be checked
    # that the ion of interest falls in wavelength
    # range of the grating

    args = parser.parse_args()
    LOGGER.info("Initialized Arguments: \n"
                "Ion: {} \n"
                "Instrument: {} \n"
                "Filetype: {} \n"
                "Velocity Window: [{}, {}] \n"
                "Grating: {} \n"
                "Redshift: {}".format(args.ion, args.instrument.upper(),
                                      args.filetype.upper(), args.vel_min,
                                      args.vel_max, args.grating.upper(),
                                      args.redshift))
    LOGGER.warning("If these are *not* the parameters you intended to use, "
                   "please perform a new measurement. Future versions of "
                   "this package will support mid process adjustments of "
                   "these parameters.")
    spectrum = build_spectrum(DATADIR, args.instrument.upper(),
                              args.filetype.upper(), args.grating.upper(),
                              args.redshift)

    return args, spectrum


# --------------------------------------------------------------------------- #


def lsr_correct(args, spectrum):
    """This function performs the lsr correction"""
    # find ion wavelength from ions.csv
    if BATCH_MODE:
        spectrum.get_ions(args["ion"])
    else:
        spectrum.get_ions(args.ion)
    # transform wavelength array of spectrum object to velocity space
    spectrum.calculate_velocity()
    # find RA & DEC of target from target list
    spectrum.get_coords(TARGETS)
    # l & b are properties of the spectrum object dependent on RA & DEC and
    # are therefore automatically available
    # calculate velocity correction & add velocity correction to spectrum
    # object wavelength array (in velocity space)
    spectrum.lsr_correct_velocity()

    return spectrum


# --------------------------------------------------------------------------- #


def continuum_fit(spectrum, left=DEFAULTS["continuum_left"],
                  right=DEFAULTS["continuum_right"]):
    """This function performs all the calculations relating to the
    continuum, as well as calculating the normalized flux
    and error arrays"""

    # find indices in spectrum.velocity corresponding to left and right
    # continuum window boundaries
    left_indices = spectrum.find_indices(left)
    right_indices = spectrum.find_indices(right)
    indices = [left + right for left, right in
               zip(left_indices, right_indices)]
    # calculating means, signal to noise, pixel size, and S/N per resel
    continuum, signalnoise, pixels = spectrum.calculate_continuum(indices)

    # calculating fits to continuum windows and updating normalized flux and
    # error attributes
    spectrum, linear_fits = spectrum.calculate_fits(continuum)

    return spectrum, left_indices, right_indices


# --------------------------------------------------------------------------- #


def measure(args, spectrum):
    # find indices in velocity window (from -100 to 100, for example)
    if BATCH_MODE:
        window = [args["vel_min"], args["vel_max"]]
    else:
        window = [args.vel_min, args.vel_max]
    indices = spectrum.find_indices(window)
    # make truncated spectrum object to perform these measurements
    helper = Helper(spectrum, indices)
    # fix negative zeroes in error
    helper.error = helper.fix_negatives(helper.error)
    # set negative values in flux to percentage of the continuum array (will
    # mark these as saturation)
    helper.flux = helper.fix_negatives(helper.flux, fixes="negatives")
    # calculate apparent optical depth and error
    helper.calculate_aod()
    # calculate apparent column density and error
    helper.calculate_acd()
    # calculate equivalent width and error
    helper.calculate_ew()
    # evaluate significance of measurement, set detection flag
    helper.significance()

    # sets measurements done in helper object in spectrum object
    spectrum.set_measurements(helper)

    return spectrum, helper


# -----------------------------------------------------------------------------


if __name__ == "__main__":
    status = main()
    sys.exit(status)



#!/usr/bin/env python

"""This module is for running the measurement process multiple times.
Requires the use of batch_table.csv"""

__author__ = "Camellia Magness"
__email__ = "cmagness@stsci.edu"

import sys
import glob
import logging

import pandas as pd
from tqdm import tqdm
from astropy import constants

from . import SETTINGS
from .measure_aod import *
from .format_data import build_spectrum
from .visualization import Visualizer

INPUTS = SETTINGS["inputs"]
DATADIR = INPUTS["datadir"]
PARAMETERS = SETTINGS["parameters"]
DEFAULTS = SETTINGS["defaults"]
LOGGER = logging.getLogger(__name__)
console = logging.StreamHandler()
console.setLevel(logging.INFO)
LOGGER.addHandler(console)

C = constants.c.to('km/s').value  # km/s

N = 3.768e14  # proportionality constant -> (m_e * c)/(pi * e**2)


# at some point need to check and see if outdir (and datadir, really too)
# exist. if not, create outdir


# --------------------------------------------------------------------------- #


def main():
    # look at the table for info-turn into df, return df
    LOGGER.info("Entering batch mode...")
    if DEFAULTS["batch_table"]:
        batch_dataframe = read_table(DEFAULTS["batch_table"])
    else:
        LOGGER.error("Please provide a path to your batch table in your "
                     "settings file if you would like to perform multiple "
                     "measurements. Exiting...")
        sys.exit()

    # use rootnames to collect the files that need to be run on
    batch_dataframe = collect_files(batch_dataframe)

    # for each file, essentially run a slightly modified version of
    # measure_aod.main
    batch_run(batch_dataframe)
    return 0


def read_table(filename):
    # why did i make this its own function lol
    return pd.read_csv(filename)


def collect_files(dataframe):
    # get the column of the basenames as a list or w/e
    rootnames = dataframe["ROOTNAME"]

    # get the files in the data directory
    all_files_in_dir = glob.glob(DATADIR + "*")

    # for each file in the column, check to see if a file matches
    batch_files = []
    for rootname in rootnames:
        for filename in all_files_in_dir:
            if rootname in filename:
                batch_files += [filename]
                break
        else:  # only gets here if it doesn't break the filename statement,
            # i.e. it didn't find a match
            LOGGER.warning("No file was found matching the rootname: {}. "
                           "Continuing...".format(rootname))

    if batch_files:
        LOGGER.info("Found {} files to measure. This might take a while."
                    .format(len(batch_files)))
        dataframe["FILENAME"] = batch_files
    else:
        LOGGER.warning("Found no files to measure. Exiting...")
        sys.exit()

    return dataframe


def batch_run(dataframe):
    # for each file in the list, do:
    for index, file_row in tqdm(dataframe.iterrows()):
        # collect the arguments
        args = {"datadir": file_row["FILENAME"],
                "ins": file_row["INSTRUMENT"].upper(),
                "file": file_row["FILETYPE"].upper(),
                "grating": file_row["GRATING"].upper(),
                "redshift": file_row["REDSHIFT"]}

        # build a spectrum object
        spectrum = build_spectrum(**args)
        spectrum.target = file_row["TARGET"]  # this is a hack to get around
        # stuff in build_spectrum
        LOGGER.info("Spectrum object successfully built.")

        # pass that on and do everything exactly the same
        # set up visualizer
        visualizer = Visualizer()
        visualizer.set_target(spectrum.target)
        visualizer.set_raw_flux(spectrum.flux)

        # LSR correction
        # need to add ion in to the args here
        args["ion"] = file_row["ION"]
        spectrum = lsr_correct(args, spectrum)
        LOGGER.info("Spectrum LSR corrected.")
        visualizer.set_raw_velocity(spectrum.raw_velocity[0])
        visualizer.set_lsr_velocity(spectrum.velocity[0])

        # continuum fit
        spectrum, left_indices, right_indices = continuum_fit(spectrum)
        LOGGER.info("Continuum fit calculated.")
        visualizer.set_contadjspec(spectrum)
        visualizer.set_indices(left_indices, right_indices)

        # measure aod/acd/ew
        # set measurements back in spectrum object from helper object
        # need to add the vel_min & max here
        args["vel_min"] = file_row["VEL_MIN"]
        args["vel_max"] = file_row["VEL_MAX"]
        spectrum, helper = measure(args, spectrum)
        visualizer.set_helper(helper)

        # generate table
        spectrum.generate_table(args["vel_min"], args["vel_max"])
        visualizer.plot()
        LOGGER.info("Finished measurements for {}"
                    .format(file_row["FILENAME"]))

    # finish entire list
    LOGGER.info("spectrAOD complete.")


# --------------------------------------------------------------------------- #


if __name__ == "__main__":
    status = main()
    sys.exit(status)



#!/usr/bin/env python

"""This module is for formatting input data into an object that stores
primarily the wavelength and flux data, as well
as any relevant target information. This object is then used by
measure_aod.py to perform the measurements."""

__author__ = "Camellia Magness"
__email__ = "cmagness@stsci.edu"

import os
import sys
import glob
import logging

import numpy as np
from astropy import constants
from astropy.io import fits
from astropy.io import ascii

from . import SETTINGS
from .spectrum_classes import X1DSpectrum, ASCIISpectrum

INPUTS = SETTINGS["inputs"]
DATADIR = INPUTS["datadir"]
PARAMETERS = SETTINGS["parameters"]
DEFAULTS = SETTINGS["defaults"]
LOGGER = logging.getLogger(__name__)
console = logging.StreamHandler()
console.setLevel(logging.INFO)
LOGGER.addHandler(console)

C = constants.c.to('km/s').value  # km/s

N = 3.768e14  # proportionality constant -> (m_e * c)/(pi * e**2)

# at some point need to check and see if outdir (and datadir, really too)
# exist. if not, create outdir
# add logger instead of print statements

# notes on class functionality:
# add functionality to build fits files as a method of class objects


# --------------------------------------------------------------------------- #


def build_spectrum(datadir=DATADIR, ins=PARAMETERS["instrument"],
                   file=PARAMETERS["filetype"], grating=PARAMETERS["grating"],
                   redshift=PARAMETERS["redshift"]):
    # this function should build and return the appropriate Spectrum object
    # for measure_aod.py
    # default inputs for instrument and file type are COS and X1D at the
    # moment, for testing

    spectrum = None
    batch_mode = DEFAULTS["batch_table"]

    # this might (?) need handling for other inputs other than just COS as
    # well. at some point.
    if ins == "COS":
        if file == "X1DSUM":
            if batch_mode:
                x1dsums = [datadir]  # this will really be a path to one file
            else:
                x1dsums = glob.glob(datadir + "*x1dsum.fits")
            for x1dsum in x1dsums:
                with fits.open(x1dsum) as f:
                    prhd = f["PRIMARY"].header
                    opt_elem = prhd["OPT_ELEM"]
                    # WHAT IF IT FINDS MULTIPLE X1DSUMS HERE?
                    if opt_elem == grating:
                        target_data = f["SCI"].data
                        target = prhd["TARGNAME"]
                        wave = np.array(target_data[
                                            "WAVELENGTH"].ravel())
                        # best way to do this, ravelling here?
                        flux = np.array(target_data["FLUX"].ravel())
                        error = np.array(target_data["ERROR"].ravel())
                        spectrum = X1DSpectrum(x1dsum, target, wave, flux,
                                               error, redshift)
        elif file == "BART":
            if batch_mode:
                asciis = [datadir]  # same story as the x1dsums
            else:
                search_string = "_spec-{}".format(grating)
                asciis = glob.glob(datadir + "*" + search_string)
            for ascii_file in asciis:
                basename = os.path.basename(ascii_file)
                # this can be done better
                target = basename.replace(search_string, "")
                data = ascii.read(ascii_file, names=["wave", "flux", "error"])
                if len(data.columns) > 3:
                    raise IndexError("Too many columns")
                    # choose a better error type?
                wave = np.array(data.columns["wave"])
                flux = np.array(data.columns["flux"])
                error = np.array(data.columns["error"])
                spectrum = ASCIISpectrum(target, wave, flux, error, redshift)
        elif file == "BART-N":
            if batch_mode:
                asciis = [datadir]  # same story as the x1dsums
            else:
                search_string = "_spec-{}-N".format(grating)
                asciis = glob.glob(datadir + "*" + search_string)
            for ascii_file in asciis:
                basename = os.path.basename(ascii_file)
                # this can be done better
                target = basename.replace(search_string, "")
                data = ascii.read(ascii_file, names=["wave", "flux", "error"])
                if len(data.columns) > 3:
                    raise IndexError("Too many columns")
                    # choose a better error type?
                wave = np.array(data.columns["wave"])
                flux = np.array(data.columns["flux"])
                error = np.array(data.columns["error"])
                spectrum = ASCIISpectrum(target, wave, flux, error, redshift)
        elif file == "ASCII":
            if batch_mode:
                asciis = [datadir]  # same story as the x1dsums
                # how do i get the target now??
                target = ""
                # ya don't, not in this hack. it'll be empty but batch mode
                # will provide it
            else:
                target = DEFAULTS["target"]
                asciis = glob.glob(datadir + "*" + target + "*")
            for ascii_file in asciis:
                data = ascii.read(ascii_file, names=["wave", "flux", "error"])
                if len(data.columns) > 3:
                    raise IndexError("Too many columns")
                    # choose a better error type?
                wave = np.array(data.columns["wave"])
                flux = np.array(data.columns["flux"])
                error = np.array(data.columns["error"])
                spectrum = ASCIISpectrum(target, wave, flux, error, redshift)
        else:
            LOGGER.warning("Other file types are not yet supported at this "
                           "time.")
    else:
        LOGGER.warning("This Instrument is not yet supported at this time.")

    if not spectrum:
        LOGGER.error("Spectrum object not built. Unable to proceed. "
                     "Exiting...")
        sys.exit()

    return spectrum


# --------------------------------------------------------------------------- #
